/***************************************************************************
                          missionselector  -  description
                             -------------------
    begin                : Fri Jan 30 2004
    copyright            : (C) 2004 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "missionselector.h"

#include <qlayout.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qtextedit.h>
#include <qstringlist.h>

#include <klocale.h>
#include <kfiledialog.h>

#include "taxipilot.h"
#include "cdp.h"

MissionSelector::MissionSelector(bool scan) : QDialog(tp) {
	QGridLayout *mainlayout = new QGridLayout (this);
	QVBoxLayout *mainvbox = new QVBoxLayout (mainlayout);
	mainvbox->setMargin (5);
	QHBoxLayout *selectionhbox = new QHBoxLayout (mainvbox);

	missionlist = new QListView (this);
	missionlist->addColumn (i18n ("Title"));
	missionlist->addColumn (i18n ("Author(s)"));
	missionlist->addColumn (i18n ("Location"));
	missionlist->setSelectionMode (QListView::Single);
	connect (missionlist, SIGNAL (selectionChanged ()), this, SLOT (selectionChanged ()));
	selectionhbox->addWidget (missionlist);

	QVBoxLayout *selectionbuttonbox = new QVBoxLayout (selectionhbox);
	QPushButton *scanbutton = new QPushButton (i18n ("(Re-)scan directory for new missions"), this);
	connect (scanbutton, SIGNAL (clicked ()), this, SLOT (scanButtonPressed ()));
	QPushButton *addbutton = new QPushButton (i18n ("Add mission(s) manually"), this);
	connect (addbutton, SIGNAL (clicked ()), this, SLOT (addButtonPressed ()));
	delbutton = new QPushButton (i18n ("Remove selected mission from list"), this);
	connect (delbutton, SIGNAL (clicked ()), this, SLOT (delButtonPressed ()));
	selectionbuttonbox->addWidget (scanbutton);
	selectionbuttonbox->addWidget (addbutton);
	selectionbuttonbox->addWidget (delbutton);
	selectionbuttonbox->setResizeMode (QLayout::Fixed);
	selectionbuttonbox->setMargin (10);

	description = new QTextEdit (this);
	mainvbox->addWidget (description);

	mainvbox->addSpacing (10);
	select_on_startup_box = new QCheckBox (i18n ("Always show this dialog on startup"), this);
	select_on_startup_box->setChecked (cdp->selectMissionOnStartup());
	mainvbox->addWidget (select_on_startup_box);
	mainvbox->addSpacing (10);

	QHBoxLayout *dialogbuttonbox = new QHBoxLayout (mainvbox);

	okbutton = new QPushButton (i18n ("Start mission"), this);
	connect (okbutton, SIGNAL (clicked ()), this, SLOT (okButtonPressed ()));
	QPushButton *cancelbutton = new QPushButton (i18n ("Cancel"), this);
	connect (cancelbutton, SIGNAL (clicked ()), this, SLOT (reject ()));
	dialogbuttonbox->addWidget (okbutton);
	dialogbuttonbox->addStretch ();
	dialogbuttonbox->addWidget (cancelbutton);

	setCaption (i18n ("Chose mission to play"));

	if (scan) {
		QString dummy = cdp->defaultPath();
		QStringList list = cdp->scanNewMissions (dummy);
		for (QStringList::Iterator it = list.begin(); it != list.end(); ++it) {
			addMission (*it);
		}
	}
	initialize ();
}

MissionSelector::~MissionSelector() {
}

void MissionSelector::addButtonPressed () {
	QStringList list = KFileDialog::getOpenFileNames(cdp->defaultPath (), "*.mission", this, i18n("Select Mission(s) to add"));

	for (QStringList::Iterator it = list.begin(); it != list.end(); ++it) {
		addMission (*it);
	}
}

void MissionSelector::scanButtonPressed () {
	QString path = KFileDialog::getExistingDirectory(cdp->defaultPath (), this, i18n("Select Directory to scan for missions"));

	QStringList list = cdp->scanNewMissions (path);
	for (QStringList::Iterator it = list.begin(); it != list.end(); ++it) {
		addMission (*it);
	}
}

void MissionSelector::delButtonPressed () {
	QListViewItem *item = missionlist->selectedItem ();

	abstracts.remove (item);
	delete item;
}

void MissionSelector::okButtonPressed () {
	cdp->known_missions.clear ();

	QListViewItem *item = missionlist->firstChild ();
	while (item) {
		cdp->known_missions.append (item->text (2));
		item = item->nextSibling ();
	}

	cdp->pref_selectMissionOnStartup = select_on_startup_box->isChecked ();
	cdp->selected_mission = missionlist->selectedItem ()->text (2);

	cdp->saveConfig ();

	accept ();
}

void MissionSelector::selectionChanged () {
	QListViewItem *item = missionlist->selectedItem ();

	if (item) {
		delbutton->setEnabled (true);
		okbutton->setEnabled (true);

		description->setText (abstracts[item]);
	} else {
		delbutton->setEnabled (false);
		okbutton->setEnabled (false);

		description->setText (i18n ("No mission selected. Click on a mission above to view a short description. Use the buttons on the top-right to add/remove missions to/from the list."));
	}
}

void MissionSelector::initialize () {
	QStringList list = cdp->getMissionList();
	for (QStringList::Iterator it = list.begin(); it != list.end(); ++it) {
		addMission (*it);
	}

	selectionChanged ();
}

void MissionSelector::addMission (const QString & filename) {
	QListViewItem *item = missionlist->firstChild ();
	while (item) {
		if (item->text (2) == filename) {		// already present
			return;
		} else {
			item = item->nextSibling ();
		}
	}

	Taxipilot::MissionDescription desc = tp->getMissionDescription (filename);

// TODO: print failure notice!
	if (!desc.ok) return;

	item = new QListViewItem (missionlist, desc.title, desc.author, filename);

	abstracts.insert (item, desc.abstract);
}

#include "missionselector.moc"
